-- crabSteerTrailer
-- Hundegang-Lenkung fr Trailer mit Lenkachsen
--
-- upsidedown 18.05.2013


crabSteerTrailer = {};

function crabSteerTrailer.prerequisitesPresent(specializations)
    --return true;
	return not SpecializationUtil.hasSpecialization(steerable, specializations)
end;

function crabSteerTrailer:load(xmlFile)
    self.changeSteer = SpecializationUtil.callSpecializationsFunction("changeSteer");
	self.crabSteerMode =0;
	
	self.HGroad = {}
	self.HGroad.Min = {}
	self.HGroad.Max = {}
	
	local maxSteer = 0;
	for k, wheel in pairs(self.wheels) do
		self.HGroad.Max[k] = wheel.steeringAxleRotMax
		if math.abs(self.HGroad.Max[k]) > maxSteer then
			maxSteer = math.abs(self.HGroad.Max[k]);
		end
		self.HGroad.Min[k] = wheel.steeringAxleRotMin	
		if math.abs(self.HGroad.Min[k]) > maxSteer then
			maxSteer = math.abs(self.HGroad.Min[k]);
		end		
	end
	self.HGdog = {};
	for k, wheel in pairs(self.wheels) do
		self.HGdog[k] = maxSteer;
	end
	self.HGdirection = 1;
end;

function crabSteerTrailer:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)  
   if not resetVehicles then
       self.crabSteerMode = Utils.getNoNil(getXMLFloat(xmlFile, key.."#steermode"),0);
	   self.HGdirection = Utils.getNoNil(getXMLFloat(xmlFile, key.."#steerDirection"),1);	   
end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function crabSteerTrailer:getSaveAttributesAndNodes(nodeIdent)
   local attributes = ' steermode="'..tonumber(self.crabSteerMode)..'" steerDirection="'..tonumber(self.HGdirection)..'"';
   return attributes, nil;
end;

function crabSteerTrailer:mouseEvent(posX, posY, isDown, isUp, button)
end;

function crabSteerTrailer:keyEvent(unicode, sym, modifier, isDown)
end;

function crabSteerTrailer:writeStream(streamId, connection)

	streamWriteInt8(streamId, self.crabSteerMode)
	streamWriteInt8(streamId, self.HGdirection)
	for k, wheel in pairs(self.wheels) do
		streamWriteFloat32(streamId, wheel.steeringAxleRotMax)
		streamWriteFloat32(streamId, wheel.steeringAxleRotMaxSoll)
		streamWriteFloat32(streamId, wheel.steeringAxleRotMin)
		streamWriteFloat32(streamId, wheel.steeringAxleRotMinSoll)
	end
end

function crabSteerTrailer:readStream(streamId, connection)  
	self.crabSteerMode = streamReadInt8(streamId);
	self.HGdirection = streamReadInt8(streamId);
	for k, wheel in pairs(self.wheels) do
		wheel.steeringAxleRotMax = streamReadFloat32(streamId);
		wheel.steeringAxleRotMaxSoll = streamReadFloat32(streamId);
		wheel.steeringAxleRotMin = streamReadFloat32(streamId);
		wheel.steeringAxleRotMinSoll = streamReadFloat32(streamId);
	end
end

function crabSteerTrailer:update(dt)
	local pushEvent = false;
	if self:getIsActiveForInput() then
		
		if InputBinding.hasEvent(InputBinding.crabSteerTrailer_Center) then
		   self.crabSteerMode = 0;
		   pushEvent = true;
		end;
		
		if InputBinding.hasEvent(InputBinding.crabSteerTrailer_Left) then
		   self.crabSteerMode = 1;
		   self.HGdirection = 1;
		   pushEvent = true;
		end;
		if InputBinding.hasEvent(InputBinding.crabSteerTrailer_Right) then
		   self.crabSteerMode = 1;
		   self.HGdirection = -1;
		   pushEvent = true;
		end;
		
		
		
		if pushEvent then --MP sync
			--print("pushEvent")
			if g_server ~= nil then
				g_server:broadcastEvent(crabSteerTrailerModeEvent:new(self, self.crabSteerMode), nil, nil, self);
                g_server:broadcastEvent(crabSteerTrailerLREvent:new(self, self.HGdirection), nil, nil, self);
            else
                g_client:getServerConnection():sendEvent(crabSteerTrailerLREvent:new(self, self.HGdirection));
				g_client:getServerConnection():sendEvent(crabSteerTrailerModeEvent:new(self, self.crabSteerMode));
            end;			
		end
		
		if self.attacherVehicle.crabSteerSync ~= nil then
			if self.attacherVehicle.crabSteerSync then
				self.attacherVehicle.XS_LR = -self.crabSteerMode*self.HGdirection;
				self.attacherVehicle.CrabSteerTrailerPing = 10;
			end
		end

	end;
	
	if self:getIsActive() then
		if self.crabSteerMode == 0  then
			self:changeSteer(self.HGroad.Min,self.HGroad.Max,1)
		elseif self.crabSteerMode == 1 then
			self:changeSteer(self.HGdog,self.HGdog,self.HGdirection)
		else
			self.crabSteerMode = 0;
		end; 

		if pushEvent then --we changed something.. close the envelope
			for k, wheel in pairs(self.wheels) do
				if wheel.steeringAxleRotMax > wheel.steeringAngle then
					wheel.steeringAxleRotMax = wheel.steeringAngle
				end
				if wheel.steeringAxleRotMin < wheel.steeringAngle then
					wheel.steeringAxleRotMin = wheel.steeringAngle
				end				
			end
		end
		
		local k
		local x
		local y
		local dy
		for k, wheel in pairs(self.wheels) do
			x = wheel.steeringAxleRotMax;		
			y = wheel.steeringAxleRotMaxSoll;
			
			dy = y-x;
			if (dy)^2 > 0.00001 then
				wheel.steeringAxleRotMax = x + (dy)*0.0004*dt;
			else
				wheel.steeringAxleRotMax = wheel.steeringAxleRotMaxSoll;
			end;
		
			x = wheel.steeringAxleRotMin;		
			y = wheel.steeringAxleRotMinSoll;
			
			dy = y-x;
			if (dy)^2 > 0.00001 then
				wheel.steeringAxleRotMin = x + (dy)*0.0004*dt;
			else
				wheel.steeringAxleRotMin = wheel.steeringAxleRotMinSoll;
			end;
			
			
			
			
		end;
	end;    
end;

function crabSteerTrailer:draw()

	g_currentMission:addHelpButtonText("MSS zentrieren", InputBinding.crabSteerTrailer_Center);
	g_currentMission:addHelpButtonText("MSS links schwenken", InputBinding.crabSteerTrailer_Left);
	g_currentMission:addHelpButtonText("MSS rechts schwenken", InputBinding.crabSteerTrailer_Right);
	
end;


function crabSteerTrailer:changeSteer(wheelMin,wheelMax,direction)
	for k, wheel in pairs(self.wheels) do
		wheel.steeringAxleRotMinSoll = wheelMin[k]*direction;
		wheel.steeringAxleRotMaxSoll = wheelMax[k]*direction;
	end 
end;

function crabSteerTrailer:delete()
	if self.attacherVehicle~= nil then
		self.attacherVehicle.hasCrabSteerTrailer = false;
	end
end;

function crabSteerTrailer:onAttach()
	self.attacherVehicle.hasCrabSteerTrailer = true;
end;

function crabSteerTrailer:onDetach()
	if self.attacherVehicle ~= nil then
		self.attacherVehicle.hasCrabSteerTrailer = false;
	end
end;
